import 'package:diameter/components/dialogs.dart';
import 'package:diameter/config.dart';
import 'package:diameter/models/log_entry.dart';
import 'package:diameter/models/log_event.dart';
import 'package:diameter/models/log_event_type.dart';
import 'package:diameter/screens/log/active_log_event_list.dart';
import 'package:diameter/screens/log/log_event_detail.dart';
import 'package:flex_color_scheme/flex_color_scheme.dart';
import 'package:flutter/material.dart';
import 'package:diameter/components/progress_indicator.dart';

class LogEventListScreen extends StatefulWidget {
  final LogEntry? logEntry;

  const LogEventListScreen({Key? key, this.logEntry}) : super(key: key);

  @override
  _LogEventListScreenState createState() => _LogEventListScreenState();
}

class _LogEventListScreenState extends State<LogEventListScreen> {
  late Future<List<LogEventType>> _logEventTypes;

  void refresh({String? message}) {
    if (widget.logEntry != null) {
      setState(() {
        widget.logEntry!.events =
            LogEvent.fetchAllForLogEntry(widget.logEntry!);
        widget.logEntry!.endedEvents =
            LogEvent.fetchAllEndedByEntry(widget.logEntry!);
      });
    }
    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
    _logEventTypes = LogEventType.fetchAll();
  }

  void handleEditAction(LogEvent event) {
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => LogEventDetailScreen(
          endLogEntry: widget.logEntry!,
          logEvent: event,
        ),
      ),
    ).then((message) => refresh(message: message));
  }

  void onDelete(LogEvent logEvent) {
    logEvent.delete().then((_) => refresh(message: 'Event deleted'));
  }

  void handleDeleteAction(LogEvent logEvent) async {
    if (showConfirmationDialogOnDelete) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onDelete(logEvent),
        message: 'Are you sure you want to delete this Event?',
      );
    } else {
      onDelete(logEvent);
    }
  }

  @override
  void initState() {
    super.initState();
    refresh();
  }

  @override
  Widget build(BuildContext context) {
    bool isNew = widget.logEntry == null;
    return SingleChildScrollView(
      padding: const EdgeInsets.only(top: 10.0),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          // TODO: add button for active events
          FutureBuilder<List<LogEvent>>(
            future: widget.logEntry!.events,
            builder: (context, snapshot) {
              return ViewWithProgressIndicator(
                snapshot: snapshot,
                child: snapshot.data == null || snapshot.data!.isEmpty
                    ? const Padding(
                        padding: EdgeInsets.all(10.0),
                        child: Text('No Events for this Log Entry'),
                      )
                    : ListBody(
                        children: [
                          FutureBuilder<List<LogEventType>>(
                              future: _logEventTypes,
                              builder: (context, types) {
                                return DataTable(
                                  columnSpacing: 10.0,
                                  showCheckboxColumn: false,
                                  rows: snapshot.data != null
                                      ? snapshot.data!.map((event) {
                                          return DataRow(
                                            cells: event.asDataTableCells([
                                              IconButton(
                                                  icon: const Icon(Icons.edit),
                                                  iconSize: 16.0,
                                                  onPressed: () =>
                                                      handleEditAction(event)),
                                              IconButton(
                                                  icon:
                                                      const Icon(Icons.delete),
                                                  iconSize: 16.0,
                                                  onPressed: () =>
                                                      handleDeleteAction(
                                                          event)),
                                            ], types: types.data),
                                          );
                                        }).toList()
                                      : [],
                                  columns: LogEvent.asDataTableColumns(),
                                );
                              })
                        ],
                      ),
              );
            },
          ),
          FutureBuilder<List<LogEvent>>(
            future: widget.logEntry!.endedEvents,
            builder: (context, snapshot) {
              return ViewWithProgressIndicator(
                snapshot: snapshot,
                child: snapshot.data == null || snapshot.data!.isEmpty
                    ? Container()
                    : ListBody(
                        children: [
                          FutureBuilder<List<LogEventType>>(
                              future: _logEventTypes,
                              builder: (context, types) {
                                return DataTable(
                                  columnSpacing: 10.0,
                                  showCheckboxColumn: false,
                                  rows: snapshot.data != null
                                      ? snapshot.data!.map((event) {
                                          return DataRow(
                                            cells: event.asDataTableCells([
                                              IconButton(
                                                  icon: const Icon(Icons.edit),
                                                  iconSize: 16.0,
                                                  onPressed: () =>
                                                      handleEditAction(event)),
                                              IconButton(
                                                  icon:
                                                      const Icon(Icons.delete),
                                                  iconSize: 16.0,
                                                  onPressed: () =>
                                                      handleDeleteAction(
                                                          event)),
                                            ], types: types.data),
                                          );
                                        }).toList()
                                      : [],
                                  columns: LogEvent.asDataTableColumns(),
                                );
                              })
                        ],
                      ),
              );
            },
          ),
        ],
      ),
    );
  }
}
